# TODO
#  . culling of entries
#  . "more info" dialog to display all info about a record
#  . hook into logging config interface so we can dynamically ask a logging
#    hub to send records to us
#  . colorisation of entries

import sys, time, socket, cPickle
import logging, logrecv
from qt import *

class TCPHandler(logrecv.LogRecordStreamHandler):
    def handle(self):
        ''' pass the pickle through Qt (it can't handle real py objects)
        '''
        while 1:
            try:
                chunk = self.connection.recv(2)
                if len(chunk) < 2:
                    break
                slen = (ord(chunk[0]) << 8) | ord(chunk[1])
                chunk = self.connection.recv(slen)
                while len(chunk) < slen:
                    chunk = chunk + self.connection.recv(slen - len(chunk))
                self.unPickle(chunk)
            except socket.error, e:
                if type(e.args) != types.TupleType:
                    raise
                else:
                    errcode = e.args[0]
                    if errcode != RESET_ERROR:
                        raise
                    break
    def unPickle(self, data):
        self.server.emit(PYSIGNAL('logRecordPickle'), (data,))

class MessageListItem(QListViewItem):
    def paintCell(self, painter, qg, column, width, align):
        c = qg.text()
        qg.setColor(QColorGroup.Text, Qt.red)
        QListViewItem.paintCell(self, painter, qg, column, width, align)
        qg.setColor(QColorGroup.Text, c)

class LogView(QWidget):
    def __init__(self, *args):
        apply(QWidget.__init__, (self,) + args)
        self.items = []
        self.resize(600,400)
        self.setCaption("Log View")
        qApp.installEventFilter(self)
        self.topLayout = QVBoxLayout(self, 10)
        self.menubar = QMenuBar(self)
        self.menubar.setSeparator(1)
        self.popup = QPopupMenu()
        self.popup.insertItem("&Quit", qApp, SLOT("quit()"), Qt.CTRL+Qt.Key_Q)
        self.menubar.insertItem("&File", self.popup)
        self.topLayout.setMenuBar(self.menubar)

        # list box
        self.listView = QListView(self)
        self.listView.addColumn('Date')
        self.listView.addColumn('Level')
        self.listView.addColumn('Message')
        self.listView.setSelectionMode( QListView.Extended )
        self.connect(self.listView, SIGNAL('clicked( QListViewItem* )'),
            self.itemSelected)
        self.topLayout.addWidget(self.listView, 1, 0)
        self.listView.setAllColumnsShowFocus(1)

        # set up a socket to listen for stuff on the TCP logging port
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.sock.bind(('localhost', logging.DEFAULT_TCP_LOGGING_PORT))
        self.sock.listen(5)
        self.notifier = QSocketNotifier(self.sock.fileno(),
            QSocketNotifier.Read, self)
        self.connect(self.notifier, SIGNAL("activated(int)"), self.netMsg)
        self.connect(self, PYSIGNAL('logRecordPickle'), self.logRecordPickle)

    def netMsg(self, *args, **kw):
        conn, address = self.sock.accept()
        handler = TCPHandler(conn, address, self)

    def logRecordPickle(self, pickle):
        obj = cPickle.loads(pickle)
        record = logging.LogRecord(None, None, "", 0, "", (), None)
        record.__dict__.update(obj)
        self.handleLogRecord(record)

    def handleLogRecord(self, record):
        item = MessageListItem(self.listView)
        self.items.append(item)
        t = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(record.created))
        item.setText(0, t)
        item.setText(1, record.level)
        item.setText(2, record.msg)
        if len(self.items) > 10:
            del self.items[0]

    def itemSelected(self, item):
        print "List item %s selected"%item

# run the app
a = QApplication(sys.argv)
w = LogView()
a.setMainWidget(w)
w.show()
a.exec_loop()

